#For all the methods Enumerable gives you:
# http://ruby-doc.org/core-2.2.3/Enumerable.html
# Per that documentation, Sequence includes an each() method

class Sequence
	def initialize(a)
		if a.is_a? Array then # overloading
			@a = a
		else
			@a = [a]
		end
	end

	def insert(elem)
		@a.push(elem)
	end

	def each()
		for i in 0...@a.size do
			yield @a[i]
		end
	end

	def to_s()
		str = "( "
		@a.each { |x| str += "#{x} "}    # what is an easier way to do this?
		str += ")\n"
		return str
	end

	include Enumerable # mixin inheritance
end

seq = Sequence.new([1,5,3,6,2,7])
puts "Here is the sequence: #{seq}"

seq.each { |x| print "#{x} "} 
print "\n"

# And here are some things we get for free:

# Sort (if elements of array implement <=>)
sorted = seq.sort
puts "Sorted sequence: #{sorted.join(' ')}"

# Map
doubled = seq.map { |x| x *= 2 }
puts "Sequence * 2 = #{doubled.join(' ')}"

# All?
puts seq.all? { |x| x % 2 == 0 }
doubled_seq = Sequence.new(doubled)
puts doubled_seq.all? { |x| x % 2 == 0 }

# Reject
not_div_by_3 = seq.reject { |x| x % 3 == 0 }
puts "All but 3 factors: #{not_div_by_3.join(' ')}"
